#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <netinet/if_ether.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <unistd.h>
#include "Defines.h"
#include "GeneralTypes.h"
#include "util.h"

/* Definies gerais. */
#define BUFFER_SIZE	2048


/* Variveis globais. */
unsigned char etherMAC[ETH_ALEN];

void printUsage(char *_programName)
{
	printf("%s -i <interface>\n", _programName);
}

BOOLEAN setPromiscuous(char *_interface, int _sock)
{
	const char *PROCEDURE = "setPromiscuous";

	struct ifreq ifr;

	/* Tenta obter as flags atuais da interface. */
	strncpy(ifr.ifr_name, _interface, strlen(_interface)+1);
	if((ioctl(_sock, SIOCGIFFLAGS, &ifr) == -1))
	{
		/* No foi possvel salvar as flags da interface. */
		inform(DEFAULT_ERROR_FILE, PROCEDURE, "Couldn't get flags of interface %s: %s", _interface, strerror(errno));
		exit(RETURN_FALSE);
	}

	/* Tenta obter o endereo MAC da interface. */
        ifr.ifr_ifindex = if_nametoindex(ifr.ifr_name);
	if((ioctl(_sock, SIOCGIFHWADDR, &ifr) == -1))
	{
		/* No foi possvel obter endereo MAC da interface. */
		inform(DEFAULT_ERROR_FILE, PROCEDURE, "Couldn't get MAC address of the interface %s: %s", _interface, strerror(errno));
		exit(RETURN_FALSE);
	}
        memcpy(etherMAC, ifr.ifr_hwaddr.sa_data, ETH_ALEN);

	/* Coloca a interface em modo promscuo. */
	ifr.ifr_flags = ifr.ifr_flags | IFF_PROMISC;
	if (ioctl(_sock, SIOCSIFFLAGS, &ifr) == -1 )
	{
		/* No foi possvel colocar a interface em modo promscuo. */
		inform(DEFAULT_ERROR_FILE, PROCEDURE, "Couldn't put interface %s in promiscuous mode: %s", _interface, strerror(errno));
		exit(RETURN_FALSE);
	}

	inform(DEFAULT_OUT_FILE, PROCEDURE, "Interface %s successfuly put in promiscuous mode.", _interface);

	return(RETURN_TRUE);
}

int createRawSocket(void)
{
	const char *PROCEDURE = "createRawSocket";

	int sock;

	/* Tenta criar um raw socket. */
	if((sock = socket(PF_PACKET, SOCK_RAW, htons(ETH_P_ALL))) < 0)
	{
		/* No foi possvel criar um raw socket. */
		inform(DEFAULT_ERROR_FILE, PROCEDURE, "Couldn't create a raw socket: %s", strerror(errno));
		exit(RETURN_FALSE);
	};
	
	return sock;
}

BOOLEAN compareMAC(unsigned char *_MAC1, unsigned char *_MAC2)
{
	if(	_MAC1[0] == _MAC2[0] &&
		_MAC1[1] == _MAC2[1] &&
		_MAC1[2] == _MAC2[2] &&
		_MAC1[3] == _MAC2[3] &&
		_MAC1[4] == _MAC2[4] &&
		_MAC1[5] == _MAC2[5])
	{
		return RETURN_TRUE;
	}
	else
	{
		return RETURN_FALSE;
	}
}

int main(int argc, char *argv[])
{
	const char *PROCEDURE = "main";

	int parameter, sock, bytesReceived;
	char *interface, buffer[BUFFER_SIZE], *p_srcIP, srcIP[20], *p_dstIP, dstIP[20];
	unsigned char broadcastMAC[ETH_ALEN];

	BOOLEAN i_parameter = RETURN_FALSE; 

	/* Cabealhos dos protocolos. */
	struct ethhdr *ether;
	struct ether_arp *arp;
	struct ip *ip; 	

	/* Verifica parmetros. */
	while ((parameter = getopt(argc, argv, "i:")) != -1)
	{
		switch(parameter)
		{
			case 'i':
				i_parameter = RETURN_TRUE;
				interface = optarg;
				break;

			case ':':
				printUsage(argv[0]);
				exit(RETURN_FAILURE);
				break;
			case '?':
				printUsage(argv[0]);
				exit(RETURN_FAILURE);
				break;
			default:
				break;
		}
	}

	if(i_parameter == RETURN_FALSE)
	{
		printUsage(argv[0]);
		exit(RETURN_FAILURE);
	}

	/* Tenta criar um raw socket. */
	sock = createRawSocket();

	/* Tenta colocar a interface em modo promscuo. */
	setPromiscuous(interface, sock); 

	/* Define endereo de broadcast do nvel 2. */
	broadcastMAC[0] = 0xFF;
	broadcastMAC[1] = 0xFF;
	broadcastMAC[2] = 0xFF;
	broadcastMAC[3] = 0xFF;
	broadcastMAC[4] = 0xFF;
	broadcastMAC[5] = 0xFF;

	/* Lao de captura dos pacotes. */
	while(1)
	{
		/* Recebe pacote. */
		bzero(&buffer, sizeof(buffer));
		bytesReceived = recvfrom(sock, buffer, sizeof(buffer), 0, NULL, NULL);
		if(bytesReceived == -1)
		{
			/* Ocorreu um erro ao receber o pacote. */
			continue;
		}

		/* Imprime total de bytes recebidos. */
		printf("Bytes received=%d\n", bytesReceived);

		/* Extrai e imprime informaes do protocolo de enlace. */
		ether = (struct ethhdr *)buffer;
		printf("Ethernet [source MAC=%.2x:%.2x:%.2x:%.2x:%.2x:%.2x, ", ether->h_source[0], ether->h_source[1], ether->h_source[2], ether->h_source[3], ether->h_source[4], ether->h_source[5]);
		printf("destination MAC=%.2x:%.2x:%.2x:%.2x:%.2x:%.2x", ether->h_dest[0], ether->h_dest[1], ether->h_dest[2], ether->h_dest[3], ether->h_dest[4], ether->h_dest[5]);
		if(compareMAC(etherMAC, ether->h_dest) == RETURN_TRUE)
		{
			printf(" (This host), ");
		}
		else if(compareMAC(broadcastMAC, ether->h_dest) == RETURN_TRUE)
		{
			printf(" (Broadcast), ");
		}
		else
		{
			printf(", ");
		}
		
		/* Verifica qual  o protocolo de rede. */
		if(ntohs(ether->h_proto) == ETH_P_IP)
		{
			/* IP. */
			printf("protocol=%s]\n", "IP");

			/* Extrai e imprime informaes do protocolo de rede. */
			ip = (struct ip *)(buffer+sizeof(struct ethhdr));
			/*
			Curiosidade: existe um problema com inet_ntoa(), imprime valores iguais para duas chamadas seguidas com parmetros diferentes (ip_src e ip_dst por exemplo), maiores informaes em http://www.developerweb.net/sock-faq/detail.php?id=188.
			*/
			p_srcIP = inet_ntoa(ip->ip_src);
			strncpy(&srcIP, p_srcIP, sizeof(srcIP));
			p_dstIP = inet_ntoa(ip->ip_dst); 
			strncpy(&dstIP, p_dstIP, sizeof(dstIP));
			printf("IP [version=%d, header length=%d, TOS=%d, total length=%d, id=%d, flags=-, offset=%d, TTL=%d, protocol=%s, checksum=%d, source=%s, destination=%s]", ip->ip_v, ip->ip_hl, ip->ip_tos, ip->ip_len, ip->ip_id, ip->ip_off, ip->ip_ttl, getprotobynumber(ip->ip_p)->p_name, ip->ip_sum, srcIP, dstIP);
		}
		else if(ntohs(ether->h_proto) == ETH_P_ARP)
		{
			/* ARP. */
			printf("protocol=%s]\n", "ARP");

			/* Extrai e imprime informaes do protocolo de rede. */
			arp = (struct ether_arp *)(buffer+sizeof(struct ethhdr));
			if(ntohs((arp->ea_hdr).ar_op) == ARPOP_REQUEST)
			{
				printf("ARP [operation=%s]", "Request");
			}
			else if(ntohs((arp->ea_hdr).ar_op) == ARPOP_REPLY)
			{
				printf("ARP [operation=%s]", "Reply");
			}
			else
			{
				printf("ARP [operation=%s]", "(Other)");
			}
		}
		else
		{
			/* Outro. */
			printf("protocol=%s]\n", "(Other)");
		}

		/* Finaliza impresso do pacote. */	
		printf("\n\n");
	}

	return RETURN_SUCCESS;
}

