#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <errno.h>
#include <dirent.h>
#include <time.h>
#include <sys/time.h>
#include "Defines.h"
#include "GeneralTypes.h"

void sfree(void **_pointer)
{
	const char 	*PROCEDURE = "sfree";
	
	if(*_pointer == NULL)
	{
		return;
	}
	
	free(*_pointer);

	*_pointer = NULL;
	
	return; 
}

long int getCurrentTimestamp(void)
{
	const char 	*PROCEDURE = "getCurrentTimestamp";
			
	struct timeval 	currentTimestamp;
	struct timezone TZ;

	/* Gets current timestamp. */
	if(gettimeofday(&currentTimestamp, &TZ) == -1)
	{
		printf("[DEBUG] %s: Error trying to return current timestamp: %s\n", PROCEDURE, strerror(errno));
		return -1;
	}

	return currentTimestamp.tv_sec;
}

BOOLEAN inform(const char *_file, const char *_identifier, const char *format, ...)
{
	const char *PROCEDURE = "inform";

	char hour[64];
	long date;
        struct tm *time_s;
	va_list ap;
	FILE *file_fd;

	/* Opens the file. */
	if((file_fd = fopen(_file, "a+")) == NULL)
	{
		printf("%s %s: Error trying to open information file: %s\n", "[DEBUG]", PROCEDURE, _file);
		fflush(NULL);

		return RETURN_FAILURE;
	}
	
	/* Gets current time. */
	date = time(NULL);
        time_s = localtime(&date);

	bzero(hour, sizeof(hour));
	strcpy(hour, asctime(time_s));	
	hour[strlen(hour)-1] = '\0';
	
	/* Puts information on the file. */
	va_start(ap, format);
	
	fprintf(file_fd, "%s %s %s: ", hour, "[DEBUG]", _identifier);	
	vfprintf(file_fd, format, ap);
	fprintf(file_fd, "\n");
	fflush(file_fd);
	
	va_end(ap);

	/* Closes the file. */
	fclose(file_fd);
	
	/* Sucess. */
	return RETURN_SUCCESS;
}

BOOLEAN debug(BOOLEAN _debug, const char *_file, const char *_identifier, const char *format, ...)
{
	const char *PROCEDURE = "debug";

	char string[1024];	
	va_list ap;

	/* Verifies debugging level. */
	if(_debug == RETURN_TRUE)
	{
		/* No debug. */
		return RETURN_SUCCESS;
	}
	
	/* Debug. */
	va_start(ap, format);
	
	vsnprintf(string, sizeof(string), format, ap);
	inform(_file, _identifier, string);	
	
	va_end(ap);

	/* Sucess. */
	return RETURN_SUCCESS;
}

